'''default settings'''

default_conf = {
    "network": "tppc",
    "production": True,
    "deck_version": 1,  # deck version
    "change": "default",
    "provider": "explorer"  # explorer, cryptoid
    }
